'************************
'File Name:  ShowData.bas
'************************

'******************************************************************************************
'This app shows how to display data in a text box and a list box using Api function calls.
'It differs from the first six programs in this series in that multiple source code files
'are used.  This file, ShowData.bas, is the main program file.  It contains all the code
'necessary to show a very simple and small Main Window containing just two buttons.  The
'top button causes another form/dialog/window to open which contains a multi-line text box
'covering the entire extent of the form.  In creating the form and making it visible an
'ODBC function, SQLDrivers() is called so as to amass multiple lines of system database
'driver info to fill the text box and allow scrolling through the data.  This is the same
'SQLDrivers() function we covered at the end of Tutorial #6.
'
'The second button on the Main Window loads another seperate form containing a list box and
'a button at the bottom.  When the user clicks the button the list box fills with a hundred
'lines of pretty boring text manufactured in a For Loop.  That is why I decided to fill the
'text box on the first form with the SQLDriver() ODBC info - to eliminate monotomy!  When
'you double click on an item in the listbox a message box pops up showing you which item
'and text string you chose.
'
'So what I'm saying here for those who want to compile and run the program in this tutorial
'is that instead of pasting one file in your programming editor as was the case with the
'previous programs, you are going to have to copy the code from four seperate posts (this
'one and the following three) into four seperate files and name them in accordance with my
'file name in each post.  That way, the compiler will be able to find them when it reads
'the following Includes just below.  This is a highly workable means of organizing a large
'programming project where each file contains code for one form/dialog in the program.  So
'with this program we have one main switchboard type form which launches the two display
'forms constituting the 'meat' of the program, so to speak.  A file with the same base part
'of the file name as this file, i.e., 'ShowData', but with the extension .inc, contains
'equates, types, and declares used by code throughout the application.  Here are the
'includes:
'******************************************************************************************
#Compile Exe  "ShowData"      'ShowData.bas
#Dim All
#Include "Win32api.inc"       'Main Windows Api Include File.  Contains equates & declares.
#Include "ShowData.inc"       'Main Include File For This App.
#Include "frmTextBox.inc"     'Form Text Box.  Code That Runs The Text Box Example
#Include "frmListBox.inc"     'Form List Box.  Code That Runs The List Box Example



'******************************************************************************************
'Function Name:  frmMainWindow_OnCreate()  Handles WM_CREATE Message For Main Window
'******************************************************************************************

'******************************************************************************************
'If you are an ex VB programmer, this is the Form_Load() event handler.  This function will
'be called when the CreateWindowEx() call is made down in WinMain() (all the way at the
'bottom -I'm a bottom-up programmer).  The main work this function does is register two window
'classes with the operating system for the two display forms in this program, and create the
'two buttons the clicking of which will respectively instantiate the form containing a text
'box in the first case and a listbox in the second case.
'******************************************************************************************
Function frmMainWindow_OnCreate(wea As WndEventArgs) As Long 'if from VB this is Form_Load()
  Local hBut As Dword,dwStyle As Dword,hIns As Dword
  Local sz1 As Asciiz*64,sz2 As Asciiz*64
  Local pCreateStruct As CREATESTRUCT Ptr
  Local winclass As WndClassEx
  Local szTextBoxDisplay As Asciiz*16
  Local  szListBoxDisplay As Asciiz*16

  pCreateStruct=wea.lParam               'When Windows sends a WM_CREATE message the lParam
  hIns=@pCreateStruct.hInstance      'parameter is a pointer to a CREATESTRUCT Type and
  sz1="View SQLDrivers Data In Text Box" 'one of the fields is the instance handle in
  sz2="View Strings Looped Into Listbox" 'WinMain()
  dwStyle=%WS_CHILD Or %WS_VISIBLE
  'Create the two buttons on the form
  hBut=CreateWindow("button",sz1,dwStyle,50,25,325,25,wea.hWnd,%IDC_BUTTON_1,hIns,0)
  hBut=CreateWindow("button",sz2,dwStyle,50,60,325,25,wea.hWnd,%IDC_BUTTON_2,hIns,0)
  'Load WndClassEx type with desired characteristics for Forms / windows
  szTextBoxDisplay="frmTextBox"   'Register frmTextBox Class
  winclass.lpszClassName=VarPtr(szTextBoxDisplay)
  winclass.lpfnWndProc=CodePtr(frmTextBox)
  winclass.hbrBackground=%COLOR_BTNFACE+1
  winclass.cbSize=SizeOf(winclass)
  winclass.style=%CS_HREDRAW Or %CS_VREDRAW
  winclass.cbClsExtra=0  : winclass.cbWndExtra=0
  winclass.hInstance=hIns
  winclass.hIcon=LoadIcon(%NULL, ByVal %IDI_APPLICATION)
  winclass.hCursor=LoadCursor(%NULL, ByVal %IDC_ARROW)
  winclass.lpszMenuName=%NULL
  Call RegisterClassEx(winclass)
  szListBoxDisplay="frmListBox"   'Register frmListBox Class
  winclass.lpszClassName=VarPtr(szListBoxDisplay)
  winclass.lpfnWndProc=CodePtr(frmListBox)
  winclass.hbrBackground=%COLOR_BTNFACE+1
  Call RegisterClassEx(winclass)

  frmMainWindow_OnCreate=0
End Function



'******************************************************************************************
'Function Name:  frmMainWindow_OnCommand()  Handles WM_COMMAND Message For Main Window
'******************************************************************************************
'
'******************************************************************************************
'Clicking a button control causes Windows to send a WM_COMMAND message to the window procedure
'of the parent of the button control.  Note when the buttons were created just above in
'frmMainWindow_OnCreate(), the eighth parameter in the CreateWindow() call made wea.hWnd (the
'handle of the Main Window) the parent of the buttons.  So when the user (you) clicks either
'button the Window Procedure for the Main Window ( frmMainWindow() ) will receive a WM_COMMAND
'message and Select Case logic will route processing to here, i.e., frmMainWindow_OnCommand().
'Just below more Select Case logic will examine the wParam parameter associated with the
'message to determine which botton was pressed, as the form contains two buttons, and either
'one could be the source of the WM_COMMAND message.  Also note in the CreateWindow() call in
'frmMainWindow_OnCreate where the buttons were created the ninth parameter was an equate
'specifying the control id for each button.  These control ids have Global scope in the
'application and are defined in ShowData.inc.  Note that ShowData.inc was included above right
'below the #Include Win32Api.inc so when the compiler hit this procedure it knew what they
'were.  The first statement in each of the cases causes the main window to become not visible.
'The next statement assigns some bit patterns to dwStyle.  This variable goes in the third
'parameter of the CreateWindow() call that will create an instance of either the form with the
'text box or the form with the list box.  If you examine the window styles applicable to the
'CreateWindow() call you'll see quite a variety and some of the styles are composite styles
'that contain groups of other commonly used window attributes.  That is the case with the
'commonly used %WS_OVERLAPPEDWINDOW style, although here I didn't think a maximize button
'made much sence, so it was easier for me to 'xor' it out than forgo use of the overlapped
'window style and list five or so styles individually.  At some point I'll cover fiddling
'with bits in another tutorial.  For now just trust me that xor-ing a bit against itself sets
'it to zero or off.  The CreateWindow() call then instantiates for each button a window of
'either the frmTextBox or frmListBox class.  Then the window is made visible.  At that point
'the window which the code in this file pertains to will still be in memory but the form
'will be invisible.  Supposing the user clicks the top button to load the frmTextBox form,
'this narritive will jump then to frmTextBox.inc which file contains code that will pertain
'to that form.  If you are reading this as a tutorial, then jump to there.
'******************************************************************************************
Function frmMainWindow_OnCommand(wea As WndEventArgs) As Long
  Local hForm As Dword, dwStyle As Dword

  Select Case LoWrd(wea.wParam)
    Case %IDC_BUTTON_1
      Call ShowWindow(wea.hWnd,%SW_HIDE)  'Hide MainWindow
      dwStyle=%WS_OVERLAPPEDWINDOW Xor %WS_MAXIMIZEBOX  'Zor out maximize button
      'The Following Call Creates The Text Box Display Form.
      hForm= _
      CreateWindowEx _
      ( _
        0, _                  'No extended styles
        "frmTextBox", _       'We registered this class in frmMainWindow_OnCreate
        "frmTextBox", _       'Caption for window
        dwStyle, _            'We set this just above.
        200,100,375,550, _    'Location/size info
        wea.hWnd, _           'Parent is this window (frmMainWindow)
        0, _                  'No menu or id
        GetWindowLong(wea.hWnd,%GWL_HINSTANCE), _   'Instance handle
        Byval 0 _             'No extra creation data.
      )
      Call ShowWindow(hForm,%SW_SHOWNORMAL)
    Case %IDC_BUTTON_2
      Call ShowWindow(wea.hWnd,%SW_HIDE)  'Hide MainWindow
      dwStyle=%WS_OVERLAPPEDWINDOW Xor %WS_MAXIMIZEBOX  'Zor out maximize button
      'The Following Call Creates The List Box Display Form.
      hForm= _
      CreateWindowEx _
      ( _
        0, _                   'No extended styles
        "frmListBox", _        'We registered this class in frmMainWindow_OnCreate
        "frmListBox", _        'Caption for window
        dwStyle, _             'We set this just above.
        150,100,525,475, _     'Location/size info
        Wea.hWnd, _            'Parent is this window (frmMainWindow)
        0, _                   'No menu or id
        GetWindowLong(wea.hWnd,%GWL_HINSTANCE), _
        Byval 0 _              'No extra creation data.
      )
      Call ShowWindow(hForm,%SW_SHOWNORMAL)
  End Select

  frmMainWindow_OnCommand=0
End Function



'******************************************************************************************
'Function Name:  frmMainWindow_OnClose()  Handles WM_CLOSE Message For Main Window
'******************************************************************************************
Function frmMainWindow_OnClose(wea As WndEventArgs) As Long
  Call PostQuitMessage(0)   'Will cause WinMain() to drop out of the message
  frmMainWindow_OnClose=0   'processing loop and the app to close.
End Function



'*********************************************************************************************************
'Function Name:  frmMainWindow  Window Procedure For Main Window
'*********************************************************************************************************

'All messages for any and all windows of the frmMainWindow class come here.  In this app there is only
'one window, and the caption of that window is ShowData.  In this procedure a variable of type WndEventArgs
'is created with static duration.  When any of the three messages that this procedure processes are received,
'the WndEventArgs variable is loaded with the data received in the function's parameters, and processing is
'routed to the correct message handling function.  Note that if a message is received other than the three
'within the Select Case statement, the message is passed to DefWindowProc().
Function frmMainWindow(ByVal hWnd As Long,ByVal wMsg As Long,ByVal wParam As Long,ByVal lParam As Long) As Long
  Local wea As WndEventArgs

  Select Case wMsg
    Case %WM_CREATE
      wea.hWnd=hWnd : wea.wParam=wParam : wea.lParam=lParam
      frmMainWindow=frmMainWindow_OnCreate(wea)
      Exit Function
    Case %WM_COMMAND
      'When a button is clicked processing is re-routed
      'to  frmMainWindow_OnCommand(wea)
      wea.hWnd=hWnd : wea.wParam=wParam : wea.lParam=lParam
      frmMainWindow=frmMainWindow_OnCommand(wea)
      Exit Function
    Case %WM_CLOSE
      wea.hWnd=hWnd : wea.wParam=wParam : wea.lParam=lParam
      frmMainWindow=frmMainWindow_OnClose(wea)
      Exit Function
  End Select

  frmMainWindow=DefWindowProc(hWnd,wMsg,wParam,lParam)
End Function



'************************************************************************************************
'This is actually the entry point of this app.  I believe this is called 'bottom up' programming
'style.  In this style each function's declaration is combined with the function's definition, so
'that seperate DECLAREs are not needed.
'************************************************************************************************
Function WinMain(ByVal hIns As Long, ByVal hPrev As Long,ByVal lpCL As Asciiz Ptr,ByVal Is As Long) As Long
  Local szAppName As Asciiz * 16,szClassName As Asciiz*16
  Local hMainWnd As Dword,dwStyle As Dword
  Local winclass As WndClassEx
  Local Msg As tagMsg

  szAppName="ShowData"
  szClassName="frmMainWindow"
  winclass.cbSize=SizeOf(winclass)
  winclass.style=%CS_HREDRAW Or %CS_VREDRAW
  winclass.lpfnWndProc=CodePtr(frmMainWindow)
  winclass.cbClsExtra=0 : winclass.cbWndExtra=0
  winclass.hInstance=hIns
  winclass.hIcon=LoadIcon(%NULL, ByVal %IDI_APPLICATION)
  winclass.hCursor=LoadCursor(%NULL, ByVal %IDC_ARROW)
  winclass.hbrBackground=%COLOR_BTNFACE+1
  winclass.lpszMenuName=%NULL
  winclass.lpszClassName=VarPtr(szClassName)
  Call RegisterClassEx(winclass)
  dwStyle=%WS_OVERLAPPEDWINDOW Xor %WS_MAXIMIZEBOX 'Maximize box won't help much.
  hMainWnd=CreateWindowEx(0,"frmMainWindow","ShowData",dwStyle,400,300,440,150,0,0,hIns,ByVal 0)
  Call ShowWindow(hMainWnd,Is)
  Call UpdateWindow(hMainWnd)
  While GetMessage(Msg,%NULL,0,0)
    TranslateMessage Msg
    DispatchMessage Msg
  Wend

  Function=msg.wParam
End Function