'**************************
'File Name:  frmTextBox.inc
'**************************

'**********************************************************************************************
'Function Name:  frmTextBox_OnCreate  Handles WM_CREATE Message For frmTextBox Window
'**********************************************************************************************

'**********************************************************************************************
'The code in this file will run when the user clicks the top button on the MainWindow form.
'The Window procedure there, fnMainWndProc(), will sort out which button was pressed, and
'an instance of a frmTextBox class will be instantiated by a CreateWindow() call in
'fnMainWndProc_OnCommand(). That CreateWindow() call will cause the Window Procedure associated
'with the frmTextBox class to be called, here frmTextBox(), and the first message will
'be the WM_CREATE message.  Select Case logic there will cause the WM_CREATE message handler
'frmTextBox_OnCreate() to be called (this function directly below).
'
'The first job this function does is get the app instance handle from window creation data.
'Windows maintains for each class a type CREATESTRUCT that is initially filled with the various
'window creation parameters that you used in the original CreateWindow() call to create the
'window.  Any of the fields/members of the type can be retrieved during the processing of the
'WM_CREATE message because the lParam parameter is set to the address in memory of where Windows
'is storing the window creation data.
'
'The next job for the function is to come up with the window styles for a multi-line text box
'that will contain scroll bars for viewing more data than can fit on one screen.  If you search
'your Windows Api Help or go to MSDN you can find all the styles applicable to text/edit boxes.
'This is certainly one of the most important if not the most important windows control, so don't
'be too surprised to find a large number of styles for this control.  Styles are a very important
'subject when it comes to creating controls on your forms/dialogs, and frequently when you can't
'seem to get the control working the right way, it is because you don't have the right
'combination of styles in your CreateWindow() call.
'
'The text box on the form is sized to fill up the entire form.  When frmTextBox_OnCreate()
'is called, a GetClientRect() Api call is made to obtain the size of the client area of the
'form/window.  After the call the Type RECT variable rc will contain in its rc.nRight and
'rc.nBottom members the right most and bottom most extremities of the window in pixels.  These
'numbers are used in the CreateWindow() call that creates the edit box for the sixth and seventh
'parameters.
'**********************************************************************************************
Function frmTextBox_OnCreate(wea As WndEventArgs) As Long
  Local hEdit As Dword,dwStyle As Dword, hIns As Dword
  Local pCreateStruct As CREATESTRUCT Ptr
  Local strText As String
  Register i As Long
  Local rc As RECT

  pCreateStruct=wea.lParam        'get app instance handle
  hIns=@pCreateStruct.hInstance   'from window creation data
  'Below are styles you need to set if you want the text box to be multi-line with scroll bars.
  dwStyle=%WS_CHILD Or %WS_VISIBLE Or %ES_MULTILINE Or %WS_VSCROLL Or %WS_HSCROLL
  Call GetClientRect(wea.hWnd,rc) 'After this call rc will contain height and width of client area.
  'This CreateWindow() call below will create a window of the 'edit' class (A text box).
  hEdit=CreateWindow("edit","",dwStyle,0,0,rc.nRight,rc.nBottom,wea.hWnd,%IDC_EDIT_BOX,hIns,0)
  If blnGetSQLDriverData(hEdit)<>%TRUE Then  'In this call I pass the handle of the edit box to
     For i = 1 To 100                        'blnGetSQLDriverData() and that function fills the
       strText=strText & Str$(i) & _         'text box with database driver data from your computer.
       " Simple Demo Showing How To Display And Scroll Text In Edit Box" & _  'If the function fails
       Chr$(13) & Chr$(10)   'the text box gets filled up with these boring and useless strings.
     Next i
     Call SetWindowText(hEdit,ByVal StrPtr(strText)) '<<<Puts data into text box
  End If

  frmTextBox_OnCreate=0
End Function



'**********************************************************************************************
'Function Name:  frmTextBox_OnDestroy  Handles WM_DESTROY Message For frmTextBox Window
'**********************************************************************************************
Function frmTextBox_OnDestroy(wea As WndEventArgs) As Long
  Call ShowWindow(wea.hWnd, %SW_HIDE)
  Call ShowWindow(FindWindowEx(ByVal 0,ByVal 0,"frmMainWindow","ShowData"),%SW_SHOWNORMAL)

  frmTextBox_OnDestroy=0
End Function



'**********************************************************************************************
'Function Name:  frmTextBox  Window Procedure For frmTextBox Window
'**********************************************************************************************
Function frmTextBox(ByVal hWnd As Long,ByVal wMsg As Long,ByVal wParam As Long,ByVal lParam As Long) As Long
  Local wea As WndEventArgs

  Select Case wMsg
    Case %WM_CREATE
      wea.hWnd=hWnd : wea.wParam=wParam : wea.lParam=lParam
      frmTextBox=frmTextBox_OnCreate(wea)
      Exit Function
    Case %WM_DESTROY
      wea.hWnd=hWnd : wea.wParam=wParam : wea.lParam=lParam
      frmTextBox=frmTextBox_OnDestroy(wea)
      Exit Function
  End Select

  frmTextBox=DefWindowProc(hWnd,wMsg,wParam,lParam)
End Function
